@@ -14,21 +14,25 @@
 import logging
 import re
 
-import botocore.exceptions
 
-from awscli.customizations.emr.exceptions import ResolveServicePrincipalError
+import botocore.exceptions
+from botocore import xform_name
+
 from awscli.customizations.commands import BasicCommand
-from awscli.customizations.emr import constants
+from awscli.customizations.emr import configutils
 from awscli.customizations.emr import emrutils
 from awscli.customizations.emr import exceptions
+from awscli.customizations.emr.command import Command
+from awscli.customizations.emr.constants import EC2
+from awscli.customizations.emr.constants import EC2_ROLE_NAME
+from awscli.customizations.emr.constants import EMR
+from awscli.customizations.emr.constants import EMR_ROLE_NAME
+from awscli.customizations.emr.exceptions import ResolveServicePrincipalError
 
 
 LOG = logging.getLogger(__name__)
 
 
-EC2_ROLE_NAME = "EMR_EC2_DefaultRole"
-EMR_ROLE_NAME = "EMR_DefaultRole"
-
 EC2_ROLE_POLICY = {
     "Statement": [
         {
@@ -99,11 +103,11 @@ def assume_role_policy(serviceprincipal):
 
 
 def get_service_principal(service, endpoint_host):
-    return service+'.'+_get_suffix(endpoint_host)
+    return service + '.' + _get_suffix(endpoint_host)
 
 
 def _get_suffix(endpoint_host):
-        return _get_suffix_from_endpoint_host(endpoint_host)
+    return _get_suffix_from_endpoint_host(endpoint_host)
 
 
 def _get_suffix_from_endpoint_host(endpoint_host):
@@ -130,12 +134,22 @@ def _get_regex_match_from_endpoint_host(endpoint_host):
     return regex_match
 
 
-class CreateDefaultRoles(BasicCommand):
+class CreateDefaultRoles(Command):
     NAME = "create-default-roles"
     DESCRIPTION = ('Creates the default IAM role ' +
                    EC2_ROLE_NAME + ' and ' +
                    EMR_ROLE_NAME + ' which can be used when'
-                   ' creating the cluster using the create-cluster command.')
+                   ' creating the cluster using the create-cluster command.\n'
+                   '\nIf you do not have a Service Role and Instance Profile '
+                   'variable set for your create-cluster command in the AWS '
+                   'CLI config file, create-default-roles will automatically '
+                   'set the values for these variables with these default '
+                   'roles. If you have already set a value for Service Role '
+                   'or Instance Profile, create-default-roles will not '
+                   'automatically set the defaults for these variables in the '
+                   'AWS CLI config file. You can view settings for variables '
+                   'in the config file using the "aws configure get" command.'
+                   '\n')
     ARG_TABLE = [
         {'name': 'iam-endpoint',
          'no_paramfile': True,
@@ -146,19 +160,18 @@ class CreateDefaultRoles(BasicCommand):
     ]
     EXAMPLES = BasicCommand.FROM_FILE('emr', 'create-default-roles.rst')
 
-    def _run_main(self, parsed_args, parsed_globals):
+    def _run_main_command(self, parsed_args, parsed_globals):
         ec2_result = None
         emr_result = None
-        self.iam = self._session.get_service('iam')
         self.iam_endpoint_url = parsed_args.iam_endpoint
-        region = self._get_region(parsed_globals)
 
-        self._check_for_iam_endpoint(region, self.iam_endpoint_url)
+        self._check_for_iam_endpoint(self.region, self.iam_endpoint_url)
         self.emr_endpoint_url = \
-            self._session.get_service('emr').get_endpoint(
-                region_name=parsed_globals.region,
+            self._session.create_client(
+                'emr',
+                region_name=self.region,
                 endpoint_url=parsed_globals.endpoint_url,
-                verify=parsed_globals.verify_ssl).host
+                verify=parsed_globals.verify_ssl).meta.endpoint_url
 
         LOG.debug('elasticmapreduce endpoint used for resolving'
                   ' service principal: ' + self.emr_endpoint_url)
@@ -171,7 +184,7 @@ class CreateDefaultRoles(BasicCommand):
             LOG.debug('Role ' + role_name + ' does not exist.'
                       ' Creating default role for EC2: ' + role_name)
             ec2_result = self._create_role_with_role_policy(
-                role_name, role_name, constants.EC2,
+                role_name, role_name, EC2,
                 emrutils.dict_to_string(EC2_ROLE_POLICY),
                 parsed_globals)
 
@@ -196,13 +209,15 @@ class CreateDefaultRoles(BasicCommand):
             LOG.debug('Role ' + role_name + ' does not exist.'
                       ' Creating default role for EMR: ' + role_name)
             emr_result = self._create_role_with_role_policy(
-                role_name, role_name, constants.EMR,
+                role_name, role_name, EMR,
                 emrutils.dict_to_string(EMR_ROLE_POLICY),
                 parsed_globals)
 
+        configutils.update_roles(self._session)
+
         emrutils.display_response(
             self._session,
-            self._session.get_service('iam').get_operation('CreateRole'),
+            'create_role',
             self._construct_result(ec2_result, emr_result),
             parsed_globals)
 
@@ -210,7 +225,7 @@ class CreateDefaultRoles(BasicCommand):
 
     def _check_for_iam_endpoint(self, region, iam_endpoint):
         try:
-            self._session.get_service('emr').get_endpoint(region)
+            self._session.create_client('emr', region)
         except botocore.exceptions.UnknownEndpointError:
             if iam_endpoint is None:
                 raise exceptions.UnknownIamEndpointError(region=region)
@@ -227,17 +242,9 @@ class CreateDefaultRoles(BasicCommand):
             self, list, response, role_policy):
         if response is not None and response[1] is not None:
             list.append({'Role': response[1]['Role'],
-                        'RolePolicy': role_policy})
+                         'RolePolicy': role_policy})
             return list
 
-    def _get_region(self, parsed_globals):
-        region = self._session.get_config_variable('region')
-
-        if parsed_globals.region is not None:
-            region = parsed_globals.region
-
-        return region
-
     def _check_if_role_exists(self, role_name, parsed_globals):
         parameters = {'RoleName': role_name}
         try:
@@ -310,7 +317,7 @@ class CreateDefaultRoles(BasicCommand):
                                  parsed_globals)
 
     def _call_iam_operation(self, operation_name, parameters, parsed_globals):
-        operation_object = self.iam.get_operation(operation_name)
-        return emrutils.call(
-            self._session, operation_object, parameters, parsed_globals.region,
-            self.iam_endpoint_url, parsed_globals.verify_ssl)
+        client = self._session.create_client(
+            'iam', self.region, self.iam_endpoint_url,
+            parsed_globals.verify_ssl)
+        return getattr(client, xform_name(operation_name))(**parameters)
