@@ -27,10 +27,10 @@ from awscli.customizations.s3.fileinfo import TaskInfo, FileInfo
 from awscli.customizations.s3.filters import create_filter
 from awscli.customizations.s3.s3handler import S3Handler, S3StreamHandler
 from awscli.customizations.s3.utils import find_bucket_key, uni_print, \
-    AppendFilter, find_dest_path_comp_key
+    AppendFilter, find_dest_path_comp_key, human_readable_size
 from awscli.customizations.s3.syncstrategy.base import MissingFileSync, \
     SizeAndLastModifiedSync, NeverSync
-
+from awscli.customizations.s3 import transferconfig
 
 
 RECURSIVE = {'name': 'recursive', 'action': 'store_true', 'dest': 'dir_op',
@@ -38,20 +38,34 @@ RECURSIVE = {'name': 'recursive', 'action': 'store_true', 'dest': 'dir_op',
                  "Command is performed on all files or objects "
                  "under the specified directory or prefix.")}
 
+
+HUMAN_READABLE = {'name': 'human-readable', 'action': 'store_true',
+                  'help_text': "Displays file sizes in human readable format."}
+
+
+SUMMARIZE = {'name': 'summarize', 'action': 'store_true',
+             'help_text': (
+                 "Displays summary information "
+                 "(number of objects, total size).")}
+
+
 DRYRUN = {'name': 'dryrun', 'action': 'store_true',
           'help_text': (
               "Displays the operations that would be performed using the "
               "specified command without actually running them.")}
 
+
 QUIET = {'name': 'quiet', 'action': 'store_true',
          'help_text': (
              "Does not display the operations performed from the specified "
              "command.")}
 
+
 FORCE = {'name': 'force', 'action': 'store_true',
          'help_text': (
              "Deletes all objects in the bucket including the bucket itself.")}
 
+
 FOLLOW_SYMLINKS = {'name': 'follow-symlinks', 'action': 'store_true',
                    'default': True, 'group_name': 'follow_symlinks',
                    'help_text': (
@@ -63,10 +77,12 @@ FOLLOW_SYMLINKS = {'name': 'follow-symlinks', 'action': 'store_true',
                        "nor ``--no-follow-symlinks`` is specifed, the default "
                        "is to follow symlinks.")}
 
+
 NO_FOLLOW_SYMLINKS = {'name': 'no-follow-symlinks', 'action': 'store_false',
                       'dest': 'follow_symlinks', 'default': True,
                       'group_name': 'follow_symlinks'}
 
+
 NO_GUESS_MIME_TYPE = {'name': 'no-guess-mime-type', 'action': 'store_false',
                       'dest': 'guess_mime_type', 'default': True,
                       'help_text': (
@@ -74,23 +90,27 @@ NO_GUESS_MIME_TYPE = {'name': 'no-guess-mime-type', 'action': 'store_false',
                           "uploaded files.  By default the mime type of a "
                           "file is guessed when it is uploaded.")}
 
+
 CONTENT_TYPE = {'name': 'content-type', 'nargs': 1,
                 'help_text': (
                     "Specify an explicit content type for this operation.  "
                     "This value overrides any guessed mime types.")}
 
+
 EXCLUDE = {'name': 'exclude', 'action': AppendFilter, 'nargs': 1,
            'dest': 'filters',
            'help_text': (
                "Exclude all files or objects from the command that matches "
                "the specified pattern.")}
 
+
 INCLUDE = {'name': 'include', 'action': AppendFilter, 'nargs': 1,
            'dest': 'filters',
            'help_text': (
                "Don't exclude files or objects "
                "in the command that match the specified pattern")}
 
+
 ACL = {'name': 'acl', 'nargs': 1,
        'choices': ['private', 'public-read', 'public-read-write',
                    'authenticated-read', 'bucket-owner-read',
@@ -102,32 +122,41 @@ ACL = {'name': 'acl', 'nargs': 1,
            "``bucket-owner-read``, ``bucket-owner-full-control`` and "
            "``log-delivery-write``.")}
 
-GRANTS = {'name': 'grants', 'nargs': '+',
-          'help_text': (
-              "Grant specific permissions to individual users or groups. You "
-              "can supply a list of grants of the form::<p/>  --grants "
-              "Permission=Grantee_Type=Grantee_ID [Permission=Grantee_Type="
-              "Grantee_ID ...]<p/>Each value contains the following elements:"
-              "<p/><ul><li><code>Permission</code> - Specifies "
-              "the granted permissions, and can be set to read, readacl, "
-              "writeacl, or full.</li><li><code>Grantee_Type</code> - "
-              "Specifies how the grantee is to be identified, and can be set "
-              "to uri, emailaddress, or id.</li><li><code>Grantee_ID</code> - "
-              "Specifies the grantee based on Grantee_Type.</li></ul>The "
-              "<code>Grantee_ID</code> value can be one of:<ul><li><b>uri</b> "
-              "- The group's URI. For more information, see "
-              '<a href="http://docs.aws.amazon.com/AmazonS3/latest/dev/ACLOverview.html#SpecifyingGrantee">'
-              "Who Is a Grantee?</a></li>"
-              "<li><b>emailaddress</b> - The account's email address.</li>"
-              "<li><b>id</b> - The account's canonical ID</li></ul>"
-              "</li></ul>"
-              "For more information on Amazon S3 access control, see "
-              '<a href="http://docs.aws.amazon.com/AmazonS3/latest/dev/UsingAuthAccess.html">Access Control</a>')}
+
+GRANTS = {
+    'name': 'grants', 'nargs': '+',
+    'help_text': (
+        'Grant specific permissions to individual users or groups. You '
+        'can supply a list of grants of the form::<p/>  --grants '
+        'Permission=Grantee_Type=Grantee_ID [Permission=Grantee_Type='
+        'Grantee_ID ...]<p/>To specify the same permission type for multiple '
+        'grantees, specify the permission as such::<p/> --grants '
+        'Permission=Grantee_Type=Grantee_ID,Grantee_Type=Grantee_ID,...'
+        '</p>Each value contains the following elements:'
+        '<p/><ul><li><code>Permission</code> - Specifies '
+        'the granted permissions, and can be set to read, readacl, '
+        'writeacl, or full.</li><li><code>Grantee_Type</code> - '
+        'Specifies how the grantee is to be identified, and can be set '
+        'to uri, emailaddress, or id.</li><li><code>Grantee_ID</code> - '
+        'Specifies the grantee based on Grantee_Type.</li></ul>The '
+        '<code>Grantee_ID</code> value can be one of:<ul><li><b>uri</b> '
+        '- The group\'s URI. For more information, see '
+        '<a href="http://docs.aws.amazon.com/AmazonS3/latest/dev/'
+        'ACLOverview.html#SpecifyingGrantee">'
+        'Who Is a Grantee?</a></li>'
+        '<li><b>emailaddress</b> - The account\'s email address.</li>'
+        '<li><b>id</b> - The account\'s canonical ID</li></ul>'
+        '</li></ul>'
+        'For more information on Amazon S3 access control, see '
+        '<a href="http://docs.aws.amazon.com/AmazonS3/latest/dev/'
+        'UsingAuthAccess.html">Access Control</a>')}
+
 
 SSE = {'name': 'sse', 'action': 'store_true',
        'help_text': (
            "Enable Server Side Encryption of the object in S3")}
 
+
 STORAGE_CLASS = {'name': 'storage-class', 'nargs': 1,
                  'choices': ['STANDARD', 'REDUCED_REDUNDANCY'],
                  'help_text': (
@@ -135,6 +164,7 @@ STORAGE_CLASS = {'name': 'storage-class', 'nargs': 1,
                      "Valid choices are: STANDARD | REDUCED_REDUNDANCY. "
                      "Defaults to 'STANDARD'")}
 
+
 WEBSITE_REDIRECT = {'name': 'website-redirect', 'nargs': 1,
                     'help_text': (
                         "If the bucket is configured as a website, "
@@ -143,16 +173,19 @@ WEBSITE_REDIRECT = {'name': 'website-redirect', 'nargs': 1,
                         "stores the value of this header in the object "
                         "metadata.")}
 
+
 CACHE_CONTROL = {'name': 'cache-control', 'nargs': 1,
                  'help_text': (
                      "Specifies caching behavior along the "
                      "request/reply chain.")}
 
+
 CONTENT_DISPOSITION = {'name': 'content-disposition', 'nargs': 1,
                        'help_text': (
                            "Specifies presentational information "
                            "for the object.")}
 
+
 CONTENT_ENCODING = {'name': 'content-encoding', 'nargs': 1,
                     'help_text': (
                         "Specifies what content encodings have been "
@@ -160,9 +193,11 @@ CONTENT_ENCODING = {'name': 'content-encoding', 'nargs': 1,
                         "mechanisms must be applied to obtain the media-type "
                         "referenced by the Content-Type header field.")}
 
+
 CONTENT_LANGUAGE = {'name': 'content-language', 'nargs': 1,
                     'help_text': ("The language the content is in.")}
 
+
 SOURCE_REGION = {'name': 'source-region', 'nargs': 1,
                  'help_text': (
                      "When transferring objects from an s3 bucket to an s3 "
@@ -173,8 +208,35 @@ SOURCE_REGION = {'name': 'source-region', 'nargs': 1,
                      "specified the region of the source will be the same "
                      "as the region of the destination bucket.")}
 
-EXPIRES = {'name': 'expires', 'nargs': 1, 'help_text': ("The date and time at "
-           "which the object is no longer cacheable.")}
+
+EXPIRES = {
+    'name': 'expires', 'nargs': 1,
+    'help_text': (
+        "The date and time at which the object is no longer cacheable.")
+}
+
+
+METADATA_DIRECTIVE = {
+    'name': 'metadata-directive', 'nargs': 1, 'choices': ['COPY', 'REPLACE'],
+    'help_text': (
+        'Specifies whether the metadata is copied from the source object '
+        'or replaced with metadata provided when copying S3 objects. '
+        'Note that if the object is copied over in parts, the source '
+        'object\'s metadata will not be copied over, no matter the value for '
+        '``--metadata-directive``, and instead the desired metadata values '
+        'must be specified as parameters on the command line. '
+        'Valid values are ``COPY`` and ``REPLACE``. If this parameter is not '
+        'specified, ``COPY`` will be used by default. If ``REPLACE`` is used, '
+        'the copied object will only have the metadata values that were'
+        ' specified by the CLI command. Note that if you are '
+        'using any of the following parameters: ``--content-type``, '
+        '``content-language``, ``--content-encoding``, '
+        '``--content-disposition``, ``-cache-control``, or ``--expires``, you '
+        'will need to specify ``--metadata-directive REPLACE`` for '
+        'non-multipart copies if you want the copied objects to have the '
+        'specified metadata values.')
+}
+
 
 INDEX_DOCUMENT = {'name': 'index-document',
                   'help_text': (
@@ -186,16 +248,19 @@ INDEX_DOCUMENT = {'name': 'index-document',
                       'images/index.html) The suffix must not be empty and '
                       'must not include a slash character.')}
 
+
 ERROR_DOCUMENT = {'name': 'error-document',
                   'help_text': (
                       'The object key name to use when '
                       'a 4XX class error occurs.')}
 
+
 ONLY_SHOW_ERRORS = {'name': 'only-show-errors', 'action': 'store_true',
                     'help_text': (
                         'Only errors and warnings are displayed. All other '
                         'output is suppressed.')}
 
+
 EXPECTED_SIZE = {'name': 'expected-size',
                  'help_text': (
                      'This argument specifies the expected size of a stream '
@@ -207,13 +272,13 @@ EXPECTED_SIZE = {'name': 'expected-size',
 
 
 PAGE_SIZE = {'name': 'page-size', 'cli_type_name': 'integer',
-			 'help_text': (
-				 'The number of results to return in each response to a list '
-				 'operation. The default value is 1000 (the maximum allowed). '
-				 'Using a lower value may help if an operation times out.')}
+             'help_text': (
+                 'The number of results to return in each response to a list '
+                 'operation. The default value is 1000 (the maximum allowed). '
+                 'Using a lower value may help if an operation times out.')}
 
 
-TRANSFER_ARGS = [DRYRUN, QUIET, RECURSIVE, INCLUDE, EXCLUDE, ACL,
+TRANSFER_ARGS = [DRYRUN, QUIET, INCLUDE, EXCLUDE, ACL,
                  FOLLOW_SYMLINKS, NO_FOLLOW_SYMLINKS, NO_GUESS_MIME_TYPE,
                  SSE, STORAGE_CLASS, GRANTS, WEBSITE_REDIRECT, CONTENT_TYPE,
                  CACHE_CONTROL, CONTENT_DISPOSITION, CONTENT_ENCODING,
@@ -221,17 +286,16 @@ TRANSFER_ARGS = [DRYRUN, QUIET, RECURSIVE, INCLUDE, EXCLUDE, ACL,
                  PAGE_SIZE]
 
 
-def get_endpoint(service, region, endpoint_url, verify):
-    return service.get_endpoint(region_name=region, endpoint_url=endpoint_url,
-                                verify=verify)
+def get_client(session, region, endpoint_url, verify):
+    return session.create_client('s3', region_name=region,
+                                 endpoint_url=endpoint_url, verify=verify)
 
 
 class S3Command(BasicCommand):
     def _run_main(self, parsed_args, parsed_globals):
-        self.service = self._session.get_service('s3')
-        self.endpoint = get_endpoint(self.service, parsed_globals.region,
-                                     parsed_globals.endpoint_url,
-                                     parsed_globals.verify_ssl)
+        self.client = get_client(self._session, parsed_globals.region,
+                                 parsed_globals.endpoint_url,
+                                 parsed_globals.verify_ssl)
 
 
 class ListCommand(S3Command):
@@ -242,13 +306,16 @@ class ListCommand(S3Command):
     USAGE = "<S3Path> or NONE"
     ARG_TABLE = [{'name': 'paths', 'nargs': '?', 'default': 's3://',
                   'positional_arg': True, 'synopsis': USAGE}, RECURSIVE,
-                 PAGE_SIZE]
+                 PAGE_SIZE, HUMAN_READABLE, SUMMARIZE]
     EXAMPLES = BasicCommand.FROM_FILE('s3/ls.rst')
 
     def _run_main(self, parsed_args, parsed_globals):
         super(ListCommand, self)._run_main(parsed_args, parsed_globals)
         self._empty_result = False
         self._at_first_page = True
+        self._size_accumulator = 0
+        self._total_objects = 0
+        self._human_readable = parsed_args.human_readable
         path = parsed_args.paths
         if path.startswith('s3://'):
             path = path[5:]
@@ -261,8 +328,10 @@ class ListCommand(S3Command):
                                              parsed_args.page_size)
         else:
             self._list_all_objects(bucket, key, parsed_args.page_size)
+        if parsed_args.summarize:
+            self._print_summary()
         if key:
-            # User specified a key to look for. We should return an rc of one   
+            # User specified a key to look for. We should return an rc of one
             # if there are no matching keys and/or prefixes or return an rc
             # of zero if there are matching keys or prefixes.
             return self._check_no_objects()
@@ -276,12 +345,11 @@ class ListCommand(S3Command):
             return 0
 
     def _list_all_objects(self, bucket, key, page_size=None):
-
-        operation = self.service.get_operation('ListObjects')
-        iterator = operation.paginate(self.endpoint, bucket=bucket,
-                                      prefix=key, delimiter='/',
+        paginator = self.client.get_paginator('list_objects')
+        iterator = paginator.paginate(Bucket=bucket,
+                                      Prefix=key, Delimiter='/',
                                       page_size=page_size)
-        for _, response_data in iterator:
+        for response_data in iterator:
             self._display_page(response_data)
 
     def _display_page(self, response_data, use_basename=True):
@@ -298,6 +366,8 @@ class ListCommand(S3Command):
             uni_print(print_str)
         for content in contents:
             last_mod_str = self._make_last_mod_str(content['LastModified'])
+            self._size_accumulator += int(content['Size'])
+            self._total_objects += 1
             size_str = self._make_size_str(content['Size'])
             if use_basename:
                 filename_components = content['Key'].split('/')
@@ -310,8 +380,7 @@ class ListCommand(S3Command):
         self._at_first_page = False
 
     def _list_all_buckets(self):
-        operation = self.service.get_operation('ListBuckets')
-        response_data = operation.call(self.endpoint)[1]
+        response_data = self.client.list_buckets()
         buckets = response_data['Buckets']
         for bucket in buckets:
             last_mod_str = self._make_last_mod_str(bucket['CreationDate'])
@@ -319,10 +388,10 @@ class ListCommand(S3Command):
             uni_print(print_str)
 
     def _list_all_objects_recursive(self, bucket, key, page_size=None):
-        operation = self.service.get_operation('ListObjects')
-        iterator = operation.paginate(self.endpoint, bucket=bucket,
-                                      prefix=key, page_size=page_size)
-        for _, response_data in iterator:
+        paginator = self.client.get_paginator('list_objects')
+        iterator = paginator.paginate(Bucket=bucket,
+                                      Prefix=key, page_size=page_size)
+        for response_data in iterator:
             self._display_page(response_data, use_basename=False)
 
     def _check_no_objects(self):
@@ -351,9 +420,24 @@ class ListCommand(S3Command):
         """
         This function creates the size string when objects are being listed.
         """
-        size_str = str(size)
+        if self._human_readable:
+            size_str = human_readable_size(size)
+        else:
+            size_str = str(size)
         return size_str.rjust(10, ' ')
 
+    def _print_summary(self):
+        """
+        This function prints a summary of total objects and total bytes
+        """
+        print_str = str(self._total_objects)
+        uni_print("\nTotal Objects: ".rjust(15, ' ') + print_str + "\n")
+        if self._human_readable:
+            print_str = human_readable_size(self._size_accumulator)
+        else:
+            print_str = str(self._size_accumulator)
+        uni_print("Total Size: ".rjust(15, ' ') + print_str + "\n")
+
 
 class WebsiteCommand(S3Command):
     NAME = 'website'
@@ -364,11 +448,10 @@ class WebsiteCommand(S3Command):
 
     def _run_main(self, parsed_args, parsed_globals):
         super(WebsiteCommand, self)._run_main(parsed_args, parsed_globals)
-        operation = self.service.get_operation('PutBucketWebsite')
         bucket = self._get_bucket_name(parsed_args.paths[0])
         website_configuration = self._build_website_configuration(parsed_args)
-        operation.call(self.endpoint, bucket=bucket,
-                       website_configuration=website_configuration)
+        self.client.put_bucket_website(
+            Bucket=bucket, WebsiteConfiguration=website_configuration)
         return 0
 
     def _build_website_configuration(self, parsed_args):
@@ -400,17 +483,20 @@ class S3TransferCommand(S3Command):
         super(S3TransferCommand, self)._run_main(parsed_args, parsed_globals)
         self._convert_path_args(parsed_args)
         params = self._build_call_parameters(parsed_args, {})
-        cmd_params = CommandParameters(self._session, self.NAME, params,
+        cmd_params = CommandParameters(self.NAME, params,
                                        self.USAGE)
         cmd_params.add_region(parsed_globals)
         cmd_params.add_endpoint_url(parsed_globals)
         cmd_params.add_verify_ssl(parsed_globals)
         cmd_params.add_page_size(parsed_args)
         cmd_params.add_paths(parsed_args.paths)
-        cmd_params.check_force(parsed_globals)
+        self._handle_rm_force(parsed_globals, cmd_params.parameters)
+        runtime_config = transferconfig.RuntimeConfig().build_config(
+            **self._session.get_scoped_config().get('s3', {}))
         cmd = CommandArchitecture(self._session, self.NAME,
-                                  cmd_params.parameters)
-        cmd.set_endpoints()
+                                  cmd_params.parameters,
+                                  runtime_config)
+        cmd.set_clients()
         cmd.create_instructions()
         return cmd.run()
 
@@ -434,6 +520,26 @@ class S3TransferCommand(S3Command):
                 new_path = enc_path.decode('utf-8')
                 parsed_args.paths[i] = new_path
 
+    def _handle_rm_force(self, parsed_globals, parameters):
+        """
+        This function recursive deletes objects in a bucket if the force
+        parameters was thrown when using the remove bucket command.
+        """
+        # XXX: This shouldn't really be here.  This was originally moved from
+        # the CommandParameters class to here, but this is still not the ideal
+        # place for this code.  This should be moved
+        # to either the CommandArchitecture class, or the RbCommand class where
+        # the actual operations against S3 are performed.  This may require
+        # some refactoring though to move this to either of those classes.
+        # For now, moving this out of CommandParameters allows for that class
+        # to be kept simple.
+        if 'force' in parameters:
+            if parameters['force']:
+                bucket = find_bucket_key(parameters['src'][5:])[0]
+                path = 's3://' + bucket
+                del_objects = RmCommand(self._session)
+                del_objects([path, '--recursive'], parsed_globals)
+
 
 class CpCommand(S3TransferCommand):
     NAME = 'cp'
@@ -442,7 +548,8 @@ class CpCommand(S3TransferCommand):
     USAGE = "<LocalPath> <S3Path> or <S3Path> <LocalPath> " \
             "or <S3Path> <S3Path>"
     ARG_TABLE = [{'name': 'paths', 'nargs': 2, 'positional_arg': True,
-                  'synopsis': USAGE}] + TRANSFER_ARGS + [EXPECTED_SIZE]
+                  'synopsis': USAGE}] + TRANSFER_ARGS + \
+                [METADATA_DIRECTIVE, EXPECTED_SIZE, RECURSIVE]
     EXAMPLES = BasicCommand.FROM_FILE('s3/cp.rst')
 
 
@@ -453,7 +560,8 @@ class MvCommand(S3TransferCommand):
     USAGE = "<LocalPath> <S3Path> or <S3Path> <LocalPath> " \
             "or <S3Path> <S3Path>"
     ARG_TABLE = [{'name': 'paths', 'nargs': 2, 'positional_arg': True,
-                  'synopsis': USAGE}] + TRANSFER_ARGS
+                  'synopsis': USAGE}] + TRANSFER_ARGS + [METADATA_DIRECTIVE,
+                                                         RECURSIVE]
     EXAMPLES = BasicCommand.FROM_FILE('s3/mv.rst')
 
 
@@ -505,27 +613,34 @@ class CommandArchitecture(object):
     lsit of instructions to wire together an assortment of generators to
     perform the command.
     """
-    def __init__(self, session, cmd, parameters):
+    def __init__(self, session, cmd, parameters, runtime_config=None):
         self.session = session
         self.cmd = cmd
         self.parameters = parameters
         self.instructions = []
-        self._service = self.session.get_service('s3')
+        self._runtime_config = runtime_config
         self._endpoint = None
         self._source_endpoint = None
+        self._client = None
+        self._source_client = None
 
-    def set_endpoints(self):
-        self._endpoint = get_endpoint(
-            self._service,
+    def set_clients(self):
+        self._client = get_client(
+            self.session,
+            region=self.parameters['region'],
+            endpoint_url=self.parameters['endpoint_url'],
+            verify=self.parameters['verify_ssl']
+        )
+        self._source_client = get_client(
+            self.session,
             region=self.parameters['region'],
             endpoint_url=self.parameters['endpoint_url'],
             verify=self.parameters['verify_ssl']
         )
-        self._source_endpoint = self._endpoint
         if self.parameters['source_region']:
             if self.parameters['paths_type'] == 's3s3':
-                self._source_endpoint = get_endpoint(
-                    self._service,
+                self._source_client = get_client(
+                    self.session,
                     region=self.parameters['source_region'][0],
                     endpoint_url=None,
                     verify=self.parameters['verify_ssl']
@@ -553,7 +668,7 @@ class CommandArchitecture(object):
             return False
         else:
             return True
-    
+
     def choose_sync_strategies(self):
         """Determines the sync strategy for the command.
 
@@ -622,21 +737,19 @@ class CommandArchitecture(object):
         }
         result_queue = queue.Queue()
         operation_name = cmd_translation[paths_type][self.cmd]
-        file_generator = FileGenerator(self._service,
-                                       self._source_endpoint,
+        file_generator = FileGenerator(self._source_client,
                                        operation_name,
                                        self.parameters['follow_symlinks'],
                                        self.parameters['page_size'],
                                        result_queue=result_queue)
-        rev_generator = FileGenerator(self._service, self._endpoint, '',
+        rev_generator = FileGenerator(self._client, '',
                                       self.parameters['follow_symlinks'],
                                       self.parameters['page_size'],
                                       result_queue=result_queue)
         taskinfo = [TaskInfo(src=files['src']['path'],
                              src_type='s3',
                              operation_name=operation_name,
-                             service=self._service,
-                             endpoint=self._endpoint)]
+                             client=self._client)]
         stream_dest_path, stream_compare_key = find_dest_path_comp_key(files)
         stream_file_info = [FileInfo(src=files['src']['path'],
                                      dest=stream_dest_path,
@@ -644,12 +757,12 @@ class CommandArchitecture(object):
                                      src_type=files['src']['type'],
                                      dest_type=files['dest']['type'],
                                      operation_name=operation_name,
-                                     service=self._service,
-                                     endpoint=self._endpoint,
+                                     client=self._client,
                                      is_stream=True)]
-        file_info_builder = FileInfoBuilder(self._service, self._endpoint,
-                                 self._source_endpoint, self.parameters) 
+        file_info_builder = FileInfoBuilder(
+            self._client, self._source_client, self.parameters)
         s3handler = S3Handler(self.session, self.parameters,
+                              runtime_config=self._runtime_config,
                               result_queue=result_queue)
         s3_stream_handler = S3StreamHandler(self.session, self.parameters,
                                             result_queue=result_queue)
@@ -668,7 +781,7 @@ class CommandArchitecture(object):
                             's3_handler': [s3handler]}
         elif self.cmd == 'cp' and self.parameters['is_stream']:
             command_dict = {'setup': [stream_file_info],
-                             's3_handler': [s3_stream_handler]}
+                            's3_handler': [s3_stream_handler]}
         elif self.cmd == 'cp':
             command_dict = {'setup': [files],
                             'file_generator': [file_generator],
@@ -712,7 +825,7 @@ class CommandArchitecture(object):
         # tasks failed and the number of tasks warned.
         # This means that files[0] now contains a namedtuple with
         # the number of failed tasks and the number of warned tasks.
-        # In terms of the RC, we're keeping it simple and saying 
+        # In terms of the RC, we're keeping it simple and saying
         # that > 0 failed tasks will give a 1 RC and > 0 warned
         # tasks will give a 2 RC.  Otherwise a RC of zero is returned.
         rc = 0
@@ -728,12 +841,16 @@ class CommandParameters(object):
     This class is used to do some initial error based on the
     parameters and arguments passed to the command line.
     """
-    def __init__(self, session, cmd, parameters, usage):
+    def __init__(self, cmd, parameters, usage):
         """
         Stores command name and parameters.  Ensures that the ``dir_op`` flag
         is true if a certain command is being used.
+
+        :param cmd: The name of the command, e.g. "rm".
+        :param parameters: A dictionary of parameters.
+        :param usage: A usage string
+
         """
-        self.session = session
         self.cmd = cmd
         self.parameters = parameters
         self.usage = usage
@@ -801,14 +918,6 @@ class CommandParameters(object):
                     path += '/'
                     paths[i] = path
 
-    def _verify_bucket_exists(self, bucket_name):
-        session = self.session
-        service = session.get_service('s3')
-        endpoint = service.get_endpoint(self.parameters['region'])
-        operation = service.get_operation('ListObjects')
-        # This will raise an exception if the bucket does not exist.
-        operation.call(endpoint, bucket=bucket_name, max_keys=0)
-
     def check_path_type(self, paths):
         """
         This initial check ensures that the path types for the specified
@@ -861,21 +970,6 @@ class CommandParameters(object):
             else:
                 raise Exception("Error: Local path does not exist")
 
-    def check_force(self, parsed_globals):
-        """
-        This function recursive deletes objects in a bucket if the force
-        parameters was thrown when using the remove bucket command.
-        """
-        if 'force' in self.parameters:
-            if self.parameters['force']:
-                bucket = find_bucket_key(self.parameters['src'][5:])[0]
-                path = 's3://' + bucket
-                try:
-                    del_objects = RmCommand(self.session)
-                    del_objects([path, '--recursive'], parsed_globals)
-                except:
-                    pass
-
     def add_region(self, parsed_globals):
         self.parameters['region'] = parsed_globals.region
 
