@@ -24,13 +24,90 @@ from dateutil.parser import parse
 from dateutil.tz import tzlocal
 from botocore.compat import unquote_str
 
-from awscli.customizations.s3.constants import MAX_PARTS
-from awscli.customizations.s3.constants import MAX_SINGLE_UPLOAD_SIZE
 from awscli.compat import six
 from awscli.compat import PY3
 from awscli.compat import queue
 
 
+HUMANIZE_SUFFIXES = ('KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB')
+MAX_PARTS = 10000
+# The maximum file size you can upload via S3 per request.
+# See: http://docs.aws.amazon.com/AmazonS3/latest/dev/UploadingObjects.html
+# and: http://docs.aws.amazon.com/AmazonS3/latest/dev/qfacts.html
+MAX_SINGLE_UPLOAD_SIZE = 5 * (1024 ** 3)
+SIZE_SUFFIX = {
+    'kb': 1024,
+    'mb': 1024 ** 2,
+    'gb': 1024 ** 3,
+    'tb': 1024 ** 4,
+    'kib': 1024,
+    'mib': 1024 ** 2,
+    'gib': 1024 ** 3,
+    'tib': 1024 ** 4,
+}
+
+
+
+def human_readable_size(value):
+    """Convert an size in bytes into a human readable format.
+
+    For example::
+
+        >>> human_readable_size(1)
+        '1 Byte'
+        >>> human_readable_size(10)
+        '10 Bytes'
+        >>> human_readable_size(1024)
+        '1.0 KiB'
+        >>> human_readable_size(1024 * 1024)
+        '1.0 MiB'
+
+    :param value: The size in bytes
+    :return: The size in a human readable format based on base-2 units.
+
+    """
+    one_decimal_point = '%.1f'
+    base = 1024
+    bytes_int = float(value)
+
+    if bytes_int == 1:
+        return '1 Byte'
+    elif bytes_int < base:
+        return '%d Bytes' % bytes_int
+
+    for i, suffix in enumerate(HUMANIZE_SUFFIXES):
+        unit = base ** (i+2)
+        if round((bytes_int / unit) * base) < base:
+            return '%.1f %s' % ((base * bytes_int / unit), suffix)
+
+
+def human_readable_to_bytes(value):
+    """Converts a human readable size to bytes.
+
+    :param value: A string such as "10MB".  If a suffix is not included,
+        then the value is assumed to be an integer representing the size
+        in bytes.
+    :returns: The converted value in bytes as an integer
+
+    """
+    value = value.lower()
+    if value[-2:] == 'ib':
+        # Assume IEC suffix.
+        suffix = value[-3:].lower()
+    else:
+        suffix = value[-2:].lower()
+    has_size_identifier = (
+        len(value) >= 2 and suffix in SIZE_SUFFIX)
+    if not has_size_identifier:
+        try:
+            return int(value)
+        except ValueError:
+            raise ValueError("Invalid size value: %s" % value)
+    else:
+        multiplier = SIZE_SUFFIX[suffix]
+        return int(value[:-len(suffix)]) * multiplier
+
+
 class AppendFilter(argparse.Action):
     """
     This class is used as an action when parsing the parameters.
@@ -188,17 +265,6 @@ def check_etag(etag, fileobj):
             raise MD5Error
 
 
-def check_error(response_data):
-    """
-    A helper function that prints out the error message recieved in the
-    response_data and raises an error when there is an error.
-    """
-    if response_data:
-        if 'Error' in response_data:
-            error = response_data['Error']
-            raise Exception("Error: %s\n" % error['Message'])
-
-
 def create_warning(path, error_message):
     """
     This creates a ``PrintTask`` for whenever a warning is to be thrown.
@@ -211,18 +277,6 @@ def create_warning(path, error_message):
     return warning_message
 
 
-def operate(service, cmd, kwargs):
-    """
-    A helper function that universally calls any command by taking in the
-    service, name of the command, and any additional parameters required in
-    the call.
-    """
-    operation = service.get_operation(cmd)
-    http_response, response_data = operation.call(**kwargs)
-    check_error(response_data)
-    return response_data, http_response
-
-
 def find_chunksize(size, current_chunksize):
     """
     The purpose of this function is determine a chunksize so that
@@ -379,27 +433,26 @@ def _date_parser(date_string):
 
 class BucketLister(object):
     """List keys in a bucket."""
-    def __init__(self, operation, endpoint, date_parser=_date_parser):
-        self._operation = operation
-        self._endpoint = endpoint
+    def __init__(self, client, date_parser=_date_parser):
+        self._client = client
         self._date_parser = date_parser
 
     def list_objects(self, bucket, prefix=None, page_size=None):
-        kwargs = {'bucket': bucket, 'encoding_type': 'url',
+        kwargs = {'Bucket': bucket, 'EncodingType': 'url',
                   'page_size': page_size}
         if prefix is not None:
-            kwargs['prefix'] = prefix
+            kwargs['Prefix'] = prefix
         # This event handler is needed because we use encoding_type url and
         # we're paginating.  The pagination token is the last Key of the
         # Contents list.  However, botocore does not know that the encoding
         # type needs to be urldecoded.
-        with ScopedEventHandler(self._operation.session,
+        with ScopedEventHandler(self._client.meta.events,
                                 'after-call.s3.ListObjects',
                                 self._decode_keys,
-                                'BucketListerDecodeKeys',
-                                True):
-            pages = self._operation.paginate(self._endpoint, **kwargs)
-            for response, page in pages:
+                                'BucketListerDecodeKeys'):
+            paginator = self._client.get_paginator('list_objects')
+            pages = paginator.paginate(**kwargs)
+            for page in pages:
                 contents = page.get('Contents', [])
                 for content in contents:
                     source_path = bucket + '/' + content['Key']
@@ -416,22 +469,19 @@ class BucketLister(object):
 class ScopedEventHandler(object):
     """Register an event callback for the duration of a scope."""
 
-    def __init__(self, session, event_name, handler, unique_id=None,
-                 unique_id_uses_count=False):
-        self._session = session
+    def __init__(self, event_emitter, event_name, handler, unique_id=None):
+        self._event_emitter = event_emitter
         self._event_name = event_name
         self._handler = handler
         self._unique_id = unique_id
-        self._unique_id_uses_count = unique_id_uses_count
 
     def __enter__(self):
-        self._session.register(self._event_name, self._handler, self._unique_id,
-                               self._unique_id_uses_count)
+        self._event_emitter.register(self._event_name, self._handler,
+                                     self._unique_id)
 
     def __exit__(self, exc_type, exc_value, traceback):
-        self._session.unregister(self._event_name, self._handler,
-                                 self._unique_id,
-                                 self._unique_id_uses_count)
+        self._event_emitter.unregister(self._event_name, self._handler,
+                                       self._unique_id)
 
 
 class PrintTask(namedtuple('PrintTask',
@@ -452,4 +502,7 @@ IORequest = namedtuple('IORequest',
                        ['filename', 'offset', 'data', 'is_stream'])
 # Used to signal that IO for the filename is finished, and that
 # any associated resources may be cleaned up.
-IOCloseRequest = namedtuple('IOCloseRequest', ['filename'])
+_IOCloseRequest = namedtuple('IOCloseRequest', ['filename', 'desired_mtime'])
+class IOCloseRequest(_IOCloseRequest):
+    def __new__(cls, filename, desired_mtime=None):
+        return super(IOCloseRequest, cls).__new__(cls, filename, desired_mtime)
