@@ -14,6 +14,10 @@
 Utility functions to make it easier to work with customizations.
 
 """
+
+from botocore.exceptions import ClientError
+
+
 def rename_argument(argument_table, existing_name, new_name):
     current = argument_table[existing_name]
     argument_table[new_name] = current
@@ -60,3 +64,17 @@ def _get_group_for_key(key, groups):
     for group in groups:
         if key in group:
             return group
+
+
+def s3_bucket_exists(s3_client, bucket_name):
+    bucket_exists = True
+    try:
+        # See if the bucket exists by running a head bucket
+        s3_client.head_bucket(Bucket=bucket_name)
+    except ClientError as e:
+        # If a client error is thrown. Check that it was a 404 error.
+        # If it was a 404 error, than the bucket does not exist.
+        error_code = int(e.response['Error']['Code'])
+        if error_code == 404:
+            bucket_exists = False
+    return bucket_exists
