@@ -52,13 +52,15 @@ from awscli.customizations.generatecliskeleton import \
     register_generate_cli_skeleton
 from awscli.customizations.assumerole import register_assume_role_provider
 from awscli.customizations.waiters import register_add_waiters
-from awscli.customizations.codedeploy import initialize as codedeploy_init
+from awscli.customizations.codedeploy.codedeploy import initialize as \
+    codedeploy_init
 from awscli.customizations.configservice.subscribe import register_subscribe
 from awscli.customizations.configservice.getstatus import register_get_status
 from awscli.customizations.configservice.rename_cmd import \
     register_rename_config
 from awscli.customizations.scalarparse import register_scalar_parser
 from awscli.customizations.opsworks import initialize as opsworks_init
+from awscli.customizations.awslambda import register_lambda_create_function
 
 
 def awscli_initialize(event_handlers):
@@ -69,7 +71,8 @@ def awscli_initialize(event_handlers):
     # generic error handler.
     register_s3_error_msg(event_handlers)
     error_handler = ErrorHandler()
-    event_handlers.register('after-call', error_handler)
+    event_handlers.register('after-call', error_handler,
+                            unique_id='awscli-error-handler')
 #    # The following will get fired for every option we are
 #    # documenting.  It will attempt to add an example_fn on to
 #    # the parameter object if the parameter supports shorthand
@@ -120,3 +123,4 @@ def awscli_initialize(event_handlers):
     register_rename_config(event_handlers)
     register_scalar_parser(event_handlers)
     opsworks_init(event_handlers)
+    register_lambda_create_function(event_handlers)
