@@ -30,9 +30,11 @@ import logging
 import tempfile
 import platform
 import contextlib
+import string
 from pprint import pformat
 from subprocess import Popen, PIPE
 
+
 try:
     import mock
 except ImportError as e:
@@ -45,6 +47,7 @@ except ImportError as e:
 from awscli.compat import six
 from botocore.hooks import HierarchicalEmitter
 from botocore.session import Session
+from botocore.exceptions import ClientError
 import botocore.loaders
 from botocore.vendored import requests
 
@@ -63,6 +66,14 @@ else:
     import unittest
 
 
+# In python 3, order matters when calling assertEqual to
+# compare lists and dictionaries with lists. Therefore,
+# assertItemsEqual needs to be used but it is renamed to
+# assertCountEqual in python 3.
+if six.PY2:
+    unittest.TestCase.assertCountEqual = unittest.TestCase.assertItemsEqual
+
+
 _LOADER = botocore.loaders.Loader()
 INTEG_LOG = logging.getLogger('awscli.tests.integration')
 AWS_CMD = None
@@ -131,6 +142,27 @@ def temporary_file(mode):
         shutil.rmtree(temporary_directory)
 
 
+def create_bucket(session, name=None, region=None):
+    """
+    Creates a bucket
+    :returns: the name of the bucket created
+    """
+    if not region:
+        region = 'us-west-2'
+    client = session.create_client('s3', region_name=region)
+    if name:
+        bucket_name = name
+    else:
+        rand1 = ''.join(random.sample(string.ascii_lowercase + string.digits,
+                                      10))
+        bucket_name = 'awscli-s3test-' + str(rand1)
+    params = {'Bucket': bucket_name}
+    if region != 'us-east-1':
+        params['CreateBucketConfiguration'] = {'LocationConstraint': region}
+    client.create_bucket(**params)
+    return bucket_name
+
+
 class BaseCLIDriverTest(unittest.TestCase):
     """Base unittest that use clidriver.
 
@@ -209,6 +241,21 @@ class CapturedRenderer(object):
         self.rendered_contents = contents.decode('utf-8')
 
 
+class CapturedOutput(object):
+    def __init__(self, stdout, stderr):
+        self.stdout = stdout
+        self.stderr = stderr
+
+
+@contextlib.contextmanager
+def capture_output():
+    stderr = six.StringIO()
+    stdout = six.StringIO()
+    with mock.patch('sys.stderr', stderr):
+        with mock.patch('sys.stdout', stdout):
+            yield CapturedOutput(stdout, stderr)
+
+
 class BaseAWSCommandParamsTest(unittest.TestCase):
     maxDiff = None
 
@@ -261,40 +308,6 @@ class BaseAWSCommandParamsTest(unittest.TestCase):
     def assert_params_for_cmd(self, cmd, params=None, expected_rc=0,
                               stderr_contains=None, ignore_params=None):
         stdout, stderr, rc = self.run_cmd(cmd, expected_rc)
-        if stderr_contains is not None:
-            self.assertIn(stderr_contains, stderr)
-        if params is not None:
-            last_params = self.last_params
-            if isinstance(last_params, dict):
-                last_params = copy.copy(self.last_params)
-                extra_params_to_ignore = ['Action', 'Version']
-                if ignore_params is None:
-                    ignore_params = extra_params_to_ignore
-                else:
-                    ignore_params.extend(extra_params_to_ignore)
-                for key in ignore_params:
-                    try:
-                        del last_params[key]
-                    except KeyError:
-                        pass
-            if params != last_params:
-                self.fail("Actual params did not match expected params.\n"
-                          "Expected:\n\n"
-                          "%s\n"
-                          "Actual:\n\n%s\n" % (
-                              pformat(params), pformat(last_params)))
-        return stdout, stderr, rc
-
-    def assert_params_for_cmd2(self, cmd, params=None, expected_rc=0,
-                               stderr_contains=None, ignore_params=None):
-        # XXX: This has a terrible name because it's intended to be
-        # temporary.  I want to switch everything off of
-        # assert_params_for_cmd and then I'll rename this to
-        # assert_params_for_cmd2.  The difference between this command
-        # and the other one is that we verify the kwargs that are sent
-        # to botocore's Operation.call(), *not* the serialized parameters
-        # onto the HTTP request.  We're one level up from that.
-        stdout, stderr, rc = self.run_cmd(cmd, expected_rc)
         if stderr_contains is not None:
             self.assertIn(stderr_contains, stderr)
         if params is not None:
@@ -329,27 +342,37 @@ class BaseAWSCommandParamsTest(unittest.TestCase):
         else:
             cmdlist = cmd
 
-        captured_stderr = six.StringIO()
-        captured_stdout = six.StringIO()
-        with mock.patch('sys.stderr', captured_stderr):
-            with mock.patch('sys.stdout', captured_stdout):
-                try:
-                    rc = self.driver.main(cmdlist)
-                except SystemExit as e:
-                    # We need to catch SystemExit so that we
-                    # can get a proper rc and still present the
-                    # stdout/stderr to the test runner so we can
-                    # figure out what went wrong.
-                    rc = e.code
-        stderr = captured_stderr.getvalue()
-        stdout = captured_stdout.getvalue()
+        with capture_output() as captured:
+            try:
+                rc = self.driver.main(cmdlist)
+            except SystemExit as e:
+                # We need to catch SystemExit so that we
+                # can get a proper rc and still present the
+                # stdout/stderr to the test runner so we can
+                # figure out what went wrong.
+                rc = e.code
+        stderr = captured.stderr.getvalue()
+        stdout = captured.stdout.getvalue()
         self.assertEqual(
             rc, expected_rc,
-            "Unexpected rc (expected: %s, actual: %s) for command: %s" % (
-                expected_rc, rc, cmd))
+            "Unexpected rc (expected: %s, actual: %s) for command: %s\n"
+            "stdout:\n%sstderr:\n%s" % (
+                expected_rc, rc, cmd, stdout, stderr))
         return stdout, stderr, rc
 
 
+class BaseAWSPreviewCommandParamsTest(BaseAWSCommandParamsTest):
+    def setUp(self):
+        self.preview_patch = mock.patch(
+            'awscli.customizations.preview.mark_as_preview')
+        self.preview_patch.start()
+        super(BaseAWSPreviewCommandParamsTest, self).setUp()
+
+    def tearDown(self):
+        self.preview_patch.stop()
+        super(BaseAWSPreviewCommandParamsTest, self).tearDown()
+
+
 class FileCreator(object):
     def __init__(self):
         self.rootdir = tempfile.mkdtemp()
@@ -496,7 +519,7 @@ def aws(command, collect_memory=False, env_vars=None,
     memory = None
     if not collect_memory:
         kwargs = {}
-        if input_data:  
+        if input_data:
             kwargs = {'input': input_data}
         stdout, stderr = process.communicate(**kwargs)
     else:
@@ -551,3 +574,123 @@ def _get_memory_with_ps(pid):
         # USER       PID  %CPU %MEM      VSZ    RSS   TT  STAT STARTED      TIME COMMAND
         # user     47102   0.0  0.1  2437000   4496 s002  S+    7:04PM   0:00.12 python2.6
         return int(stdout.splitlines()[1].split()[5]) * 1024
+
+
+class BaseS3CLICommand(unittest.TestCase):
+    """Base class for aws s3 command.
+
+    This contains convenience functions to make writing these tests easier
+    and more streamlined.
+
+    """
+    def setUp(self):
+        self.files = FileCreator()
+        self.session = botocore.session.get_session()
+        self.regions = {}
+        self.region = 'us-west-2'
+        self.client = self.session.create_client('s3', region_name=self.region)
+        self.extra_setup()
+
+    def extra_setup(self):
+        # Subclasses can use this to define extra setup steps.
+        pass
+
+    def tearDown(self):
+        self.files.remove_all()
+        self.extra_teardown()
+
+    def extra_teardown(self):
+        # Subclasses can use this to define extra teardown steps.
+        pass
+
+    def assert_key_contents_equal(self, bucket, key, expected_contents):
+        if isinstance(expected_contents, six.BytesIO):
+            expected_contents = expected_contents.getvalue().decode('utf-8')
+        actual_contents = self.get_key_contents(bucket, key)
+        # The contents can be huge so we try to give helpful error messages
+        # without necessarily printing the actual contents.
+        self.assertEqual(len(actual_contents), len(expected_contents))
+        if actual_contents != expected_contents:
+            self.fail("Contents for %s/%s do not match (but they "
+                      "have the same length)" % (bucket, key))
+
+    def create_bucket(self, name=None, region=None):
+        if not region:
+            region = self.region
+        bucket_name = create_bucket(self.session, name, region)
+        self.regions[bucket_name] = region
+        self.addCleanup(self.delete_bucket, bucket_name)
+        return bucket_name
+
+    def put_object(self, bucket_name, key_name, contents='', extra_args=None):
+        client = self.session.create_client(
+            's3', region_name=self.regions[bucket_name])
+        call_args = {
+            'Bucket': bucket_name,
+            'Key': key_name, 'Body': contents
+        }
+        if extra_args is not None:
+            call_args.update(extra_args)
+        response = client.put_object(**call_args)
+        self.addCleanup(self.delete_key, bucket_name, key_name)
+
+    def delete_bucket(self, bucket_name):
+        self.remove_all_objects(bucket_name)
+        client = self.session.create_client(
+            's3', region_name=self.regions[bucket_name])
+        response = client.delete_bucket(Bucket=bucket_name)
+        del self.regions[bucket_name]
+
+    def remove_all_objects(self, bucket_name):
+        client = self.session.create_client(
+            's3', region_name=self.regions[bucket_name])
+        paginator = client.get_paginator('list_objects')
+        pages = paginator.paginate(Bucket=bucket_name)
+        key_names = []
+        for page in pages:
+            key_names += [obj['Key'] for obj in page.get('Contents', [])]
+        for key_name in key_names:
+            self.delete_key(bucket_name, key_name)
+
+    def delete_key(self, bucket_name, key_name):
+        client = self.session.create_client(
+            's3', region_name=self.regions[bucket_name])
+        response = client.delete_object(Bucket=bucket_name, Key=key_name)
+
+    def get_key_contents(self, bucket_name, key_name):
+        client = self.session.create_client(
+            's3', region_name=self.regions[bucket_name])
+        response = client.get_object(Bucket=bucket_name, Key=key_name)
+        return response['Body'].read().decode('utf-8')
+
+    def key_exists(self, bucket_name, key_name):
+        client = self.session.create_client(
+            's3', region_name=self.regions[bucket_name])
+        try:
+            client.head_object(Bucket=bucket_name, Key=key_name)
+            return True
+        except ClientError:
+            return False
+
+    def list_buckets(self):
+        response = self.client.list_buckets()
+        return response['Buckets']
+
+    def content_type_for_key(self, bucket_name, key_name):
+        parsed = self.head_object(bucket_name, key_name)
+        return parsed['ContentType']
+
+    def head_object(self, bucket_name, key_name):
+        client = self.session.create_client(
+            's3', region_name=self.regions[bucket_name])
+        response = client.head_object(Bucket=bucket_name, Key=key_name)
+        return response
+
+    def assert_no_errors(self, p):
+        self.assertEqual(
+            p.rc, 0,
+            "Non zero rc (%s) received: %s" % (p.rc, p.stdout + p.stderr))
+        self.assertNotIn("Error:", p.stderr)
+        self.assertNotIn("failed:", p.stderr)
+        self.assertNotIn("client error", p.stderr)
+        self.assertNotIn("server error", p.stderr)
